@@ -1,18 +1,17 @@
+package local::lib;
+use 5.006;
 use strict;
 use warnings;
-
-package local::lib;
-
-use 5.006;
-
-use File::Spec ();
 use Config;
+use File::Spec ();
 
-our $VERSION = '2.000004'; # 2.0.4
+our $VERSION = '2.000006'; # 2.0.6
 $VERSION = eval $VERSION;
 
 sub import {
   my ($class, @args) = @_;
+  push @args, @ARGV
+    if $0 eq '-';
 
   my @steps;
   my %opts;
@@ -100,7 +99,6 @@ sub libs { $_[0]->{libs}   ||= [ \'PERL5LIB' ] }
 sub bins { $_[0]->{bins}   ||= [ \'PATH' ] }
 sub roots { $_[0]->{roots} ||= [ \'PERL_LOCAL_LIB_ROOT' ] }
 sub extra { $_[0]->{extra} ||= {} }
-sub shelltype { $_[0]->{shelltype} ||= $_[0]->guess_shelltype }
 sub no_create { $_[0]->{no_create} }
 
 my $_archname = $Config{archname};
@@ -335,11 +333,12 @@ sub environment_vars_string {
 
   my $build_method = "build_${shelltype}_env_declaration";
 
+  my $extra = $self->extra;
   my @envs = (
     PATH                => $self->bins,
     PERL5LIB            => $self->libs,
     PERL_LOCAL_LIB_ROOT => $self->roots,
-    %{$self->extra},
+    map { $_ => $extra->{$_} } sort keys %$extra,
   );
   my $out = '';
   while (@envs) {
@@ -362,7 +361,7 @@ sub environment_vars_string {
     }
     $out .= $self->$build_method($name, $value);
   }
-  my $wrap_method = 'wrap_' . $self->shelltype . '_output';
+  my $wrap_method = "wrap_${shelltype}_output";
   if ($self->can($wrap_method)) {
     return $self->$wrap_method($out);
   }
@@ -380,7 +379,7 @@ sub build_bourne_env_declaration {
   $value =~ s/(^|\G|$_path_sep)\$$name$_path_sep/$1\$$name\${$name+$_path_sep}/g;
   $value =~ s/$_path_sep\$$name$/\${$name+$_path_sep}\$$name/;
 
-  qq{${name}="$value";\nexport ${name};\n}
+  qq{${name}="$value"; export ${name};\n}
 }
 
 sub build_csh_env_declaration {
@@ -440,6 +439,16 @@ sub wrap_powershell_output {
   return $out || " \n";
 }
 
+sub build_fish_env_declaration {
+  my ($class, $name, $args) = @_;
+  my $value = $class->_interpolate($args, '$%s', '"', '\\%s');
+  if (!defined $value) {
+    return qq{set -e $name;\n};
+  }
+  $value =~ s/$_path_sep/ /g;
+  qq{set -x $name $value;\n};
+}
+
 sub _interpolate {
   my ($class, $args, $var_pat, $escape, $escape_pat) = @_;
   return
@@ -498,11 +507,11 @@ sub resolve_empty_path {
 
 sub resolve_home_path {
   my ($class, $path) = @_;
-  return $path unless ($path =~ /^~/);
-  my ($user) = ($path =~ /^~([^\/]+)/); # can assume ^~ so undef for 'us'
+  $path =~ /^~([^\/]*)/ or return $path;
+  my $user = $1;
   my $homedir = do {
-    if (!defined $user && defined $ENV{HOME}) {
-      $ENV{HOME}
+    if (! length($user) && defined $ENV{HOME}) {
+      $ENV{HOME};
     }
     else {
       require File::Glob;
@@ -543,11 +552,11 @@ sub ensure_dir_structure_for {
 sub guess_shelltype {
   my $shellbin
     = defined $ENV{SHELL}
-      ? (File::Spec->splitpath($ENV{SHELL}))[-1]
+      ? ($ENV{SHELL} =~ /([\w.]+)$/)[-1]
     : ( $^O eq 'MSWin32' && exists $ENV{'!EXITCODE'} )
       ? 'bash'
     : ( $^O eq 'MSWin32' && $ENV{PROMPT} && $ENV{COMSPEC} )
-      ? (File::Spec->splitpath($ENV{COMSPEC}))[-1]
+      ? ($ENV{COMSPEC} =~ /([\w.]+)$/)[-1]
     : ( $^O eq 'MSWin32' && !$ENV{PROMPT} )
       ? 'powershell.exe'
     : 'sh';
@@ -555,6 +564,7 @@ sub guess_shelltype {
   for ($shellbin) {
     return
         /csh$/                   ? 'csh'
+      : /fish/                   ? 'fish'
       : /command(?:\.com)?$/i    ? 'cmd'
       : /cmd(?:\.exe)?$/i        ? 'cmd'
       : /4nt(?:\.exe)?$/i        ? 'cmd'
@@ -591,10 +601,11 @@ From the shell -
 
   # Just print out useful shell commands
   $ perl -Mlocal::lib
-  export PERL_MB_OPT='--install_base /home/username/perl5'
-  export PERL_MM_OPT='INSTALL_BASE=/home/username/perl5'
-  export PERL5LIB="/home/username/perl5/lib/perl5"
-  export PATH="/home/username/perl5/bin:$PATH"
+  PERL_MB_OPT='--install_base /home/username/perl5'; export PERL_MB_OPT;
+  PERL_MM_OPT='INSTALL_BASE=/home/username/perl5'; export PERL_MM_OPT;
+  PERL5LIB="/home/username/perl5/lib/perl5"; export PERL5LIB;
+  PATH="/home/username/perl5/bin:$PATH"; export PATH;
+  PERL_LOCAL_LIB_ROOT="/home/usename/perl5:$PERL_LOCAL_LIB_ROOT"; export PERL_LOCAL_LIB_ROOT;
 
 From a .bashrc file -
 
